@@ -1,22 +1,22 @@
 import ipaddress
 import random
 
-
-def from_db(d):
-    nb = NetBlock(d['ipmanager.v1']['ipblock'])
-    for addr in d['ipmanager.v1']['in_use']:
-        nb.reserve(addr)
-    return nb
+from shakenfist import db
+from shakenfist import exceptions
 
 
-class NetBlock(object):
-    def __init__(self, ipblock):
+# NOTE(mikal): IPManager should _always_ return addresses as strings,
+# not as ipaddress.IPv4Address.
+
+class IPManager(object):
+    def __init__(self, uuid, ipblock, in_use=None):
+        self.uuid = uuid
         self.ipblock = ipblock
         self.ipblock_obj = ipaddress.ip_network(ipblock, strict=False)
 
         self.netmask = self.ipblock_obj.netmask
-        self.broadcast_address = self.ipblock_obj.broadcast_address
-        self.network_address = self.ipblock_obj.network_address
+        self.broadcast_address = str(self.ipblock_obj.broadcast_address)
+        self.network_address = str(self.ipblock_obj.network_address)
 
         self.num_addresses = self.ipblock_obj.num_addresses
         self.in_use = {
@@ -25,8 +25,44 @@ class NetBlock(object):
         }
         self.in_use_counter = 0
 
+        if in_use:
+            for addr in in_use:
+                self.reserve(addr)
+
+    @staticmethod
+    def new(uuid, ipblock):
+        with db.get_lock('ipmanager', None, uuid, ttl=120,
+                         op='Network object initialization'):
+            ipm = IPManager(uuid, ipblock)
+            # Reserve first IP address
+            ipm.reserve(ipm.get_address_at_index(1))
+            ipm.persist()
+        return ipm
+
+    @staticmethod
+    def from_db(uuid):
+        db_data = db.get_ipmanager(uuid)
+        ipm = IPManager(uuid=uuid, **db_data['ipmanager.v1'])
+        return ipm
+
+    def persist(self):
+        in_use = set()
+        for ip in self.in_use:
+            in_use.add(str(ip))
+
+        d = {
+            'ipmanager.v1': {
+                'ipblock': self.ipblock,
+                'in_use': list(in_use)
+            }
+        }
+        db.persist_ipmanager(self.uuid, d)
+
+    def delete(self):
+        db.delete_ipmanager(self.uuid)
+
     def get_address_at_index(self, idx):
-        return self.ipblock_obj[idx]
+        return str(self.ipblock_obj[idx])
 
     def is_in_range(self, address):
         return ipaddress.ip_address(address) in self.ipblock_obj
@@ -72,16 +108,4 @@ class NetBlock(object):
 
                 idx += 1
 
-    def save(self):
-        in_use = []
-        for ip in self.in_use:
-            ip_str = str(ip)
-            if ip_str not in in_use:
-                in_use.append(ip_str)
-
-        return {
-            'ipmanager.v1': {
-                'ipblock': self.ipblock,
-                'in_use': in_use
-            }
-        }
+        raise exceptions.CongestedNetwork('No free addresses on network')
