@@ -28,6 +28,18 @@ class FakeNetwork(object):
         self.broadcast = '127.255.255.255'
 
 
+class FakeNetworkInterface(object):
+    object_type = 'networkinterface'
+
+    def __init__(self, values):
+        self.uuid = values['uuid']
+        self.instance_uuid = values['instance_uuid']
+        self.network_uuid = values['network_uuid']
+        self.macaddr = values['macaddr']
+        self.ipv4 = values['ipv4']
+        self.order = values['order']
+
+
 class VirtMetaTestCase(test_shakenfist.ShakenFistTestCase):
     def setUp(self):
         super(VirtMetaTestCase, self).setUp()
@@ -206,24 +218,6 @@ class InstanceTestCase(test_shakenfist.ShakenFistTestCase):
         self.assertTrue(time.time() - etcd_write[1][1].update_time < 3)
         self.assertEqual('preflight', etcd_write[1][1].value)
 
-    @mock.patch('shakenfist.virt.Instance.error',
-                new_callable=mock.PropertyMock)
-    @mock.patch('shakenfist.db.get_lock')
-    @mock.patch('shakenfist.virt.Instance._db_get_attribute',
-                side_effect=[
-                    {'value': None, 'update_time': 0},
-                    {'value': 'created', 'update_time': 1},
-                ])
-    @mock.patch('shakenfist.virt.Instance._db_set_attribute')
-    @mock.patch('shakenfist.etcd.put')
-    def test_set_state_duplicate(
-            self, mock_put, mock_attribute_set, mock_state_get, mock_lock,
-            mock_error):
-        i = self._make_instance()
-        with testtools.ExpectedException(exceptions.InvalidStateException):
-            i.state = 'created'
-        self.assertEqual(2, mock_attribute_set.call_count)
-
     @mock.patch('shakenfist.db.get_lock')
     @mock.patch('shakenfist.virt.Instance._db_get_attribute',
                 side_effect=[
@@ -374,33 +368,31 @@ class InstanceTestCase(test_shakenfist.ShakenFistTestCase):
 
     # create, delete
 
-    @mock.patch('shakenfist.net.Network.from_db',
-                return_value=FakeNetwork())
-    @mock.patch('shakenfist.db.get_instance_interfaces',
+    @mock.patch('shakenfist.networkinterface.NetworkInterfaces',
                 return_value=[
-                    {
+                    FakeNetworkInterface({
                         'uuid': 'ifaceuuid',
                         'instance_uuid': 'instuuid',
                         'network_uuid': 'netuuid',
                         'macaddr': '1a:91:64:d2:15:39',
                         'ipv4': '127.0.0.5',
                         'order': 0
-                    },
-                    {
+                    }),
+                    FakeNetworkInterface({
                         'uuid': 'ifaceuuid2',
                         'instance_uuid': 'instuuid',
                         'network_uuid': 'netuuid',
                         'macaddr': '1a:91:64:d2:15:40',
                         'ipv4': '127.0.0.6',
                         'order': 1
-                    }
+                    })
                 ])
     @mock.patch('shakenfist.net.Network.from_db',
                 return_value=FakeNetwork())
     @mock.patch('shakenfist.baseobject.DatabaseBackedObject.state',
                 new_callable=mock.PropertyMock)
     def test_make_config_drive(self, mock_update, mock_net_from_db,
-                               mock_interfaces, mock_network):
+                               mock_interfaces):
         i = self._make_instance()
 
         (fd, cd_file) = tempfile.mkstemp()
@@ -442,7 +434,7 @@ class InstanceTestCase(test_shakenfist.ShakenFistTestCase):
                                 {
                                     "ethernet_mac_address": "1a:91:64:d2:15:39",
                                     "id": "eth0",
-                                    "mtu": 1450,
+                                    "mtu": 7950,
                                     "name": "eth0",
                                     "type": "vif",
                                     "vif_id": "ifaceuuid"
@@ -450,7 +442,7 @@ class InstanceTestCase(test_shakenfist.ShakenFistTestCase):
                                 {
                                     "ethernet_mac_address": "1a:91:64:d2:15:40",
                                     "id": "eth1",
-                                    "mtu": 1450,
+                                    "mtu": 7950,
                                     "name": "eth1",
                                     "type": "vif",
                                     "vif_id": "ifaceuuid2"
@@ -458,19 +450,23 @@ class InstanceTestCase(test_shakenfist.ShakenFistTestCase):
                             ],
                             "networks": [
                                 {
-                                    "id": "netuuid",
-                                    "ip_address": "127.0.0.5",
-                                    "link": "eth0",
-                                    "netmask": "255.0.0.0",
-                                    "network_id": "netuuid",
-                                    "routes": [
-                                        {
-                                            "gateway": "127.0.0.1",
-                                            "netmask": "0.0.0.0",
-                                            "network": "0.0.0.0"
-                                        }
-                                    ],
-                                    "type": "ipv4"
+                                    'id': 'netuuid-0',
+                                    'ip_address': '127.0.0.5',
+                                          'link': 'eth0',
+                                          'netmask': '255.0.0.0',
+                                          'network_id': 'netuuid',
+                                          'routes': [{'gateway': '127.0.0.1',
+                                                      'netmask': '0.0.0.0',
+                                                      'network': '0.0.0.0'}],
+                                          'type': 'ipv4'
+                                },
+                                {
+                                    'id': 'netuuid-1',
+                                    'ip_address': '127.0.0.6',
+                                    'link': 'eth1',
+                                    'netmask': '255.0.0.0',
+                                    'network_id': 'netuuid',
+                                    'type': 'ipv4'
                                 }
                             ],
                             "services": [
